@@ -1,15 +1,14 @@
 package Dancer;
-
+BEGIN {
+  $Dancer::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: lightweight yet powerful web application framework
+$Dancer::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
 use Cwd 'realpath';
 
-our $VERSION   = '1.3111_02';
-our $AUTHORITY = 'SUKRIA';
-
-$VERSION = eval $VERSION;
-
 use Dancer::App;
 use Dancer::Config;
 use Dancer::Cookies;
@@ -77,6 +76,7 @@ our @EXPORT    = qw(
   mime
   options
   param
+  param_array
   params
   pass
   path
@@ -179,6 +179,16 @@ sub mime            { Dancer::MIME->instance() }
 sub options         { Dancer::App->current->registry->universal_add('options', @_) }
 sub params          { Dancer::SharedData->request->params(@_) }
 sub param           { params->{$_[0]} }
+sub param_array     { 
+    my $value = param(shift);
+
+    my @array = ref $value eq 'ARRAY' ? @$value 
+              : defined $value        ? ( $value ) 
+              :                         ()
+              ;
+
+    return @array;
+}
 sub pass            { Dancer::SharedData->response->pass(1);
                       # throw a special continuation exception
                       Dancer::Continuation::Route::Passed->new->throw;
@@ -329,7 +339,7 @@ sub _init_script_dir {
     Dancer::Logger::core("initializing appdir to: `$appdir'");
 
     Dancer::setting(confdir => $ENV{DANCER_CONFDIR}
-      || $appdir);
+      || $appdir) unless Dancer::setting('confdir');
 
     Dancer::setting(public => $ENV{DANCER_PUBLIC}
       || Dancer::FileUtils::path($appdir, 'public'));
@@ -405,55 +415,59 @@ sub _send_file {
         }
     }
 
-    if (exists($options{filename})) {
-        $resp->push_header('Content-Disposition' => 
-            "attachment; filename=\"$options{filename}\""
-        );
-    }
+    if ($resp) {
 
-    if ( $options{'streaming'} ) {
-        # handle streaming
-        $resp->streamed( sub {
-            my ( $status, $headers ) = @_;
-            my %callbacks = defined $options{'callbacks'} ?
-                            %{ $options{'callbacks'} }    :
-                            ();
+        if (exists($options{filename})) {
+            $resp->push_header('Content-Disposition' => 
+                "attachment; filename=\"$options{filename}\""
+            );
+        }
 
-            return sub {
-                my $respond = shift;
-                exists $callbacks{'override'}
-                    and return $callbacks{'override'}->( $respond, $resp );
+        if ( $options{'streaming'} ) {
+            # handle streaming
+            $resp->streamed( sub {
+                my ( $status, $headers ) = @_;
+                my %callbacks = defined $options{'callbacks'} ?
+                                %{ $options{'callbacks'} }    :
+                                ();
 
-                # get respond callback and set headers, get writer in return
-                my $writer = $respond->( [
-                    $status,
-                    $headers,
-                ] );
+                return sub {
+                    my $respond = shift;
+                    exists $callbacks{'override'}
+                        and return $callbacks{'override'}->( $respond, $resp );
 
-                # get content from original response
-                my $content = $resp->content;
+                    # get respond callback and set headers, get writer in return
+                    my $writer = $respond->( [
+                        $status,
+                        $headers,
+                    ] );
 
-                exists $callbacks{'around'}
-                    and return $callbacks{'around'}->( $writer, $content );
+                    # get content from original response
+                    my $content = $resp->content;
 
-                if ( ref $content ) {
-                    my $bytes = $options{'bytes'} || '43008'; # 42K (dams)
-                    my $buf;
-                    while ( ( my $read = sysread $content, $buf, $bytes ) != 0 ) {
-                        if ( exists $callbacks{'around_content'} ) {
-                            $callbacks{'around_content'}->( $writer, $buf );
-                        } else {
-                            $writer->write($buf);
+                    exists $callbacks{'around'}
+                        and return $callbacks{'around'}->( $writer, $content );
+
+                    if ( ref $content ) {
+                        my $bytes = $options{'bytes'} || '43008'; # 42K (dams)
+                        my $buf;
+                        while ( ( my $read = sysread $content, $buf, $bytes ) != 0 ) {
+                            if ( exists $callbacks{'around_content'} ) {
+                                $callbacks{'around_content'}->( $writer, $buf );
+                            } else {
+                                $writer->write($buf);
+                            }
                         }
+                    } else {
+                        $writer->write($content);
                     }
-                } else {
-                    $writer->write($content);
-                }
-            };
-        } );
-    }
+                };
+            } );
+        }
 
-    return $resp if $resp;
+        return $resp;
+
+    }
 
     Dancer::Error->new(
         code    => 404,
@@ -480,15 +494,23 @@ sub _start {
     return $handler->dance;
 }
 
+
 1;
+
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer - lightweight yet powerful web application framework
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     #!/usr/bin/perl
@@ -541,7 +563,6 @@ involving Dancer and Plack, see L<Dancer::Deployment>.
 You can find out more about the many useful plugins available for Dancer in
 L<Dancer::Plugins>.
 
-
 =head1 EXPORTS
 
 By default, C<use Dancer> exports all the functions below plus sets up
@@ -611,7 +632,6 @@ them:
 
 The above would import all keywords as normal, with the exception of C<session>.
 
-
 =head1 FUNCTIONS
 
 =head2 after
@@ -657,7 +677,6 @@ In the case you have stored something else than a Scalar in your cookie:
         return ($values{token}, $values{token_secret});
     };
 
-
 =head2 cookie
 
 Accesses a cookie value (or sets it). Note that this method will
@@ -720,7 +739,7 @@ Returns the version of Dancer. If you need the major version, do something like:
 Logs a message of debug level:
 
     debug "This is a debug message";
-    
+
 See L<Dancer::Logger> for details on how to configure where log messages go.
 
 =head2 dirname
@@ -776,6 +795,8 @@ B<POST>. That is because C<forward> chains the same type of route the user
 reached. If it was a B<GET>, it will remain a B<GET> (but if you do need to
 change the method, you can do so; read on below for details.)
 
+B<WARNING> : using forward will B<not> preserve session data set on
+the forwarding rule.
 
 B<WARNING> : Issuing a forward immediately exits the current route,
 and perform the forward. Thus, any code after a forward is ignored, until the
@@ -811,7 +832,7 @@ only used to change the method of your request. Use with caution.
 
 =head2 from_dumper ($structure)
 
-Deserializes a Data::Dumper structure.
+Deserializes a L<< Data::Dumper >> structure.
 
 =head2 from_json ($structure, %options)
 
@@ -839,7 +860,6 @@ Defines a route for HTTP B<GET> requests to the given path:
 
 Note that a route to match B<HEAD> requests is automatically created as well.
 
-
 =head2 halt
 
 Sets a response object with the content given.
@@ -902,7 +922,8 @@ Adds a hook at some position. For example :
   };
 
 There can be multiple hooks assigned to a given position, and each will be
-executed in order.
+executed in order. Note that B<all> hooks are always called, even if they
+are defined in a different package loaded via C<load_app>.
 
 (For details on how to register new hooks from within plugins, see
 L<Dancer::Hook>.)
@@ -927,7 +948,6 @@ This hook receives as argument the path of the file to render.
     ...
   };
 
-
 =item before_error_init
 
 This hook receives as argument a L<Dancer::Error> object.
@@ -937,7 +957,6 @@ This hook receives as argument a L<Dancer::Error> object.
     ...
   };
 
-
 =item before_error_render
 
 This hook receives as argument a L<Dancer::Error> object.
@@ -1081,6 +1100,16 @@ argument a L<Dancer::Exception> object.
     my $exception = shift;
   };
 
+=item on_reset_state
+
+This hook is called when global state is reset to process a new request.
+It receives a boolean value that indicates whether the reset was called
+as part of a forwarded request.
+
+  hook on_reset_state => sub {
+    my $is_forward = shift;
+  };
+
 =item on_route_exception
 
 This hook is called when an exception has been caught, at the route level, just
@@ -1167,8 +1196,11 @@ commonly-used methods are summarized below:
 =head2 params
 
 I<This method should be called from a route handler>.
-It's an alias for the L<Dancer::Request params accessor|Dancer::Request/"params">. It returns
-an hash reference to all defined parameters. Check C<param> below to access quickly to a single
+It's an alias for the L<Dancer::Request params
+accessor|Dancer::Request/"params-source">. In list context it returns a
+list of key/value pair of all defined parameters. In scalar context, 
+it returns a hash reference instead.
+Check C<param> below to access quickly to a single
 parameter value.
 
 =head2 param
@@ -1182,6 +1214,21 @@ This method is an accessor to the parameters hash table.
        # ...
    }
 
+=head2 param_array
+
+I<This method should be called from a route handler>.
+Like I<param>, but always returns the parameter value or values as a list.
+Returns the number of values in scalar context.
+
+    # if request is '/tickets?tag=open&tag=closed&order=desc'...
+    get '/tickets' => sub {
+        my @tags = param_array 'tag';  # ( 'open', 'closed' )
+        my $tags = param 'tag';        # array ref
+
+        my @order = param_array 'order';  # ( 'desc' )
+        my $order = param 'order';        # 'desc'
+    };
+
 =head2 pass
 
 I<This method should be called from a route handler>.
@@ -1219,7 +1266,6 @@ you can find at:
 
 L<https://github.com/bestpractical/http-server-simple/pull/1>
 
-
 =head2 path
 
 Concatenates multiple paths together, without worrying about the underlying
@@ -1356,14 +1402,13 @@ instance, to use a custom layout:
 
 Returns a L<Dancer::Request> object representing the current request.
 
-See the L<Dancer::Request> documention for the methods you can call, for
+See the L<Dancer::Request> documentation for the methods you can call, for
 example:
 
     request->referer;         # value of the HTTP referer header
     request->remote_address;  # user's IP address
     request->user_agent;      # User-Agent header value
 
-
 =head2 send_error
 
 Returns a HTTP error.  By default the HTTP code returned is 500:
@@ -1487,7 +1532,6 @@ You can also set the number of bytes that will be read at a time (default being
         );
     };
 
-
 The content-type will be set depending on the current MIME types definition
 (see C<mime> if you want to define your own).
 
@@ -1517,7 +1561,7 @@ Note that Dancer is unable to guess the content type from the data
 contents. Therefore you might need to set the C<content_type>
 properly. For this kind of usage an attribute named C<filename> can be
 useful.  It is used as the Content-Disposition header, to hint the
-brower about the filename it should use.
+browser about the filename it should use.
 
    return send_file( \$data, content_type => 'image/png'
                              filename     => 'onion.png' );
@@ -1606,6 +1650,14 @@ If you need to fetch the session ID being used for any reason:
 
     my $id = session->id;
 
+In order to be able to use sessions, first  you need to enable session support in
+one of the configuration files.  A quick way to do it is to add
+
+    session: "YAML"
+
+to config.yml.
+
+For more details, see L<Dancer::Session>.
 
 =head2 splat
 
@@ -1689,7 +1741,7 @@ build a response with the current template engine:
     };
 
 Note that C<template> simply returns the content, so when you use it in a route
-handler, if execution of the the route handler should stop at that point, make
+handler, if execution of the route handler should stop at that point, make
 sure you use 'return' to ensure your route handler returns the content.
 
 Since template just returns the result of rendering the template, you can also
@@ -1740,7 +1792,7 @@ L<Dancer::Template::Abstract> for further details.
 
 =head2 to_dumper ($structure)
 
-Serializes a structure with Data::Dumper.
+Serializes a structure with L<< Data::Dumper >>.
 
 =head2 to_json ($structure, %options)
 
@@ -1826,7 +1878,6 @@ versions:
         "i don't want to $$value_for{action} the $$value_for{object} $$value_for{id} !"
     };
 
-
 =head2 var
 
 Provides an accessor for variables shared between filters and route handlers.
@@ -1877,7 +1928,6 @@ contribute).
 Also, why not L<watch the repo|https://github.com/PerlDancer/Dancer/toggle_watch>
 to keep up to date with the latest upcoming changes?
 
-
 =head1 GETTING HELP / CONTRIBUTING
 
 The Dancer development team can be found on #dancer on irc.perl.org:
@@ -1893,7 +1943,6 @@ L<http://lists.preshweb.co.uk/mailman/listinfo/dancer-users>
 If you'd like to contribute to the Dancer project, please see
 L<http://www.perldancer.org/contribute> for all the ways you can help!
 
-
 =head1 DEPENDENCIES
 
 The following modules are mandatory (Dancer cannot run without them):
@@ -1928,13 +1977,6 @@ The following modules are optional:
 
 =back
 
-
-=head1 LICENSE
-
-This module is free software and is published under the same
-terms as Perl itself.
-
-
 =head1 SEE ALSO
 
 Main Dancer web site: L<http://perldancer.org/>.
@@ -1942,4 +1984,15 @@ Main Dancer web site: L<http://perldancer.org/>.
 The concept behind this module comes from the Sinatra ruby project,
 see L<http://www.sinatrarb.com/> for details.
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
