@@ -1,5 +1,8 @@
 package Dancer::App;
-
+BEGIN {
+  $Dancer::App::AUTHORITY = 'cpan:SUKRIA';
+}
+$Dancer::App::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
@@ -9,6 +12,7 @@ use Dancer::Config;
 use Dancer::ModuleLoader;
 use Dancer::Route::Registry;
 use Dancer::Logger;
+use Dancer::Exception qw(:all);
 
 Dancer::App->attributes(qw(name app_prefix prefix registry settings on_lexical_prefix));
 
@@ -34,12 +38,26 @@ sub set_app_prefix {
     $self->prefix($prefix);
 }
 
+sub get_prefix {
+    # return the current prefix (if undefined, return an empty string)
+    return Dancer::App->current->prefix || '';
+}
+
+sub incr_lexical_prefix {
+    no warnings;  # for undefined
+    $_[0]->on_lexical_prefix( $_[0]->on_lexical_prefix + 1 );
+}
+
+sub dec_lexical_prefix {
+    $_[0]->on_lexical_prefix( $_[0]->on_lexical_prefix - 1 );
+}
+
 sub set_prefix {
     my ($self, $prefix, $cb) = @_;
 
     undef $prefix if defined($prefix) and $prefix eq "/";
 
-    croak "not a valid prefix: `$prefix', must start with a /"
+    raise core_app => "not a valid prefix: `$prefix', must start with a /"
       if defined($prefix) && $prefix !~ /^\//;
 
     my $app_prefix = defined $self->app_prefix ? $self->app_prefix : "";
@@ -54,10 +72,10 @@ sub set_prefix {
     }
 
     if (ref($cb) eq 'CODE') {
-        Dancer::App->current->on_lexical_prefix(1);
+        Dancer::App->current->incr_lexical_prefix;
         eval { $cb->() };
         my $e = $@;
-        Dancer::App->current->on_lexical_prefix(0);
+        Dancer::App->current->dec_lexical_prefix;
         Dancer::App->current->prefix($previous);
         die $e if $e;
     }
@@ -122,7 +140,7 @@ sub find_route {
     # if route cache is enabled, we check if we handled this path before
     if (Dancer::Config::setting('route_cache')) {
         my $route = Dancer::Route::Cache->get->route_from_path($method,
-            $request->path_info);
+            $request->path_info, $self->name);
 
         # NOTE maybe we should cache the match data as well
         if ($route) {
@@ -142,7 +160,7 @@ sub find_route {
             # if we have a route cache, store the result
             if (Dancer::Config::setting('route_cache')) {
                 Dancer::Route::Cache->get->store_path($method,
-                    $request->path_info => $r);
+                    $request->path_info => $r, $self->name);
             }
 
             return $r;
@@ -155,7 +173,7 @@ sub init {
     my ($self) = @_;
     $self->name('main') unless defined $self->name;
 
-    croak "an app named '" . $self->name . "' already exists"
+    raise core_app => "an app named '" . $self->name . "' already exists"
       if exists $_apps->{$self->name};
 
     # default values for properties
@@ -221,3 +239,30 @@ sub _set_settings {
 
 
 1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Dancer::App
+
+=head1 VERSION
+
+version 1.3131_0
+
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
