@@ -1,15 +1,22 @@
 package Dancer::Config;
-
+BEGIN {
+  $Dancer::Config::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT:  how to configure Dancer to suit your needs
+$Dancer::Config::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use base 'Exporter';
 use vars '@EXPORT_OK';
 
+use Hash::Merge::Simple;
+use Dancer::Config::Object 'hashref_to_object';
 use Dancer::Deprecation;
 use Dancer::Template;
 use Dancer::ModuleLoader;
 use Dancer::FileUtils 'path';
 use Carp;
+use Dancer::Exception qw(:all);
 
 use Encode;
 
@@ -26,17 +33,21 @@ sub settings {$SETTINGS}
 my $setters = {
     logger => sub {
         my ($setting, $value) = @_;
+        require Dancer::Logger;
         Dancer::Logger->init($value, settings());
     },
     log_file => sub {
+        require Dancer::Logger;
         Dancer::Logger->init(setting("logger"), setting());
     },
     session => sub {
         my ($setting, $value) = @_;
+        require Dancer::Session;
         Dancer::Session->init($value, settings());
     },
     template => sub {
         my ($setting, $value) = @_;
+        require Dancer::Template;
         Dancer::Template->init($value, settings());
     },
     route_cache => sub {
@@ -49,30 +60,23 @@ my $setters = {
         require Dancer::Serializer;
         Dancer::Serializer->init($value);
     },
+    # This setting has been deprecated in favor of global_warnings.
     import_warnings => sub {
+        my ($setting, $value) = @_;
+
+         Dancer::Deprecation->deprecated(
+             message => "import_warnings has been deprecated, please use global_warnings instead."
+         );
+
+        $^W = $value ? 1 : 0;
+    },
+    global_warnings => sub {
         my ($setting, $value) = @_;
         $^W = $value ? 1 : 0;
     },
-    auto_page => sub {
-        my ($setting, $auto_page) = @_;
-        if ($auto_page) {
-            require Dancer::App;
-            Dancer::App->current->registry->universal_add(
-                'get', '/:page',
-                sub {
-                    my $params = Dancer::SharedData->request->params;
-                    if  (-f Dancer::engine('template')->view($params->{page})) {
-                        return Dancer::template($params->{'page'});
-                    } else {
-                        return Dancer::pass();
-                    }
-                }
-            );
-        }
-    },
     traces => sub {
         my ($setting, $traces) = @_;
-        $Carp::Verbose = $traces ? 1 : 0;
+        $Dancer::Exception::Verbose = $traces ? 1 : 0;
     },
 };
 $setters->{log_path} = $setters->{log_file};
@@ -84,7 +88,7 @@ my $normalizers = {
           or return $charset;
         my $encoding = Encode::find_encoding($charset);
         defined $encoding
-          or croak "Charset defined in configuration is wrong : couldn't identify '$charset'";
+          or raise core_config => "Charset defined in configuration is wrong : couldn't identify '$charset'";
         my $name = $encoding->name;
         # Perl makes a distinction between the usual perl utf8, and the strict
         # utf8 charset. But we don't want to make this distinction
@@ -159,7 +163,11 @@ sub conffile { path(setting('confdir') || setting('appdir'), 'config.yml') }
 
 sub environment_file {
     my $env = setting('environment');
-    return path(setting('appdir'), 'environments', "$env.yml");
+    # XXX for compatibility reason, we duplicate the code from `init_envdir` here
+    # we don't know how if some application don't already do some weird stuff like
+    # the test in `t/15_plugins/02_config.t`.
+    my $envdir = setting('envdir') || path(setting('appdir'), 'environments');
+    return path($envdir, "$env.yml");
 }
 
 sub init_confdir {
@@ -167,30 +175,54 @@ sub init_confdir {
     setting confdir => $ENV{DANCER_CONFDIR} || setting('appdir');
 }
 
+sub init_envdir {
+    return setting('envdir') if setting('envdir');
+    my $appdirpath = defined setting('appdir')                 ?
+                     path( setting('appdir'), 'environments' ) :
+                     path('environments');
+
+    setting envdir => $ENV{DANCER_ENVDIR} || $appdirpath;
+}
+
 sub load {
     init_confdir();
+    init_envdir();
 
     # look for the conffile
     return 1 unless -f conffile;
 
     # load YAML
-    confess "Configuration file found but YAML is not installed"
-      unless Dancer::ModuleLoader->load('YAML');
+    my $module = $SETTINGS->{engines}{YAML}{module} || 'YAML';
 
-    if (!$_LOADED{conffile()}) {
+    my ( $result, $error ) = Dancer::ModuleLoader->load($module);
+    confess "Configuration file found but could not load $module: $error"
+        unless $result;
+
+    unless ($_LOADED{conffile()}) {
         load_settings_from_yaml(conffile);
         $_LOADED{conffile()}++;
     }
 
     my $env = environment_file;
-    if (-f $env && !$_LOADED{$env}) {
-        load_settings_from_yaml($env);
-        $_LOADED{$env}++;
+
+    # don't load the same env twice
+    unless( $_LOADED{$env} ) {
+        if (-f $env ) {
+            load_settings_from_yaml($env);
+            $_LOADED{$env}++;
+        }
+        elsif (setting('require_environment')) {
+            # failed to load the env file, and the main config said we needed it.
+            confess "Could not load environment file '$env', and require_environment is set";
+        }
     }
 
     foreach my $key (grep { $setters->{$_} } keys %$SETTINGS) {
         $setters->{$key}->($key, $SETTINGS->{$key});
     }
+    if ( $SETTINGS->{strict_config} ) {
+        $SETTINGS = hashref_to_object($SETTINGS);
+    }
 
     return 1;
 }
@@ -198,46 +230,38 @@ sub load {
 sub load_settings_from_yaml {
     my ($file) = @_;
 
-    my $config;
+    my $config = eval { YAML::LoadFile($file) }
+        or confess "Unable to parse the configuration file: $file: $@";
 
-    eval { $config = YAML::LoadFile($file) };
-    if (my $err = $@ || (!$config)) {
-        confess "Unable to parse the configuration file: $file: $@";
-    }
+    $SETTINGS = Hash::Merge::Simple::merge( $SETTINGS, {
+        map {
+            $_ => Dancer::Config->normalize_setting( $_, $config->{$_} )
+        } keys %$config
+    } );
 
-    for my $key (keys %{$config}) {
-        if ($MERGEABLE{$key}) {
-            my $setting = setting($key);
-            $setting->{$_} = $config->{$key}{$_} for keys %{$config->{$key}};
-        }
-        else {
-            _set_setting($key, $config->{$key});
-        }
-    }
-
-    return scalar(keys %$config);
+    return scalar keys %$config;
 }
 
 sub load_default_settings {
-    $SETTINGS->{server}       ||= $ENV{DANCER_SERVER}       || '0.0.0.0';
-    $SETTINGS->{port}         ||= $ENV{DANCER_PORT}         || '3000';
-    $SETTINGS->{content_type} ||= $ENV{DANCER_CONTENT_TYPE} || 'text/html';
-    $SETTINGS->{charset}      ||= $ENV{DANCER_CHARSET}      || '';
-    $SETTINGS->{startup_info} ||= $ENV{DANCER_STARTUP_INFO} || 1;
-    $SETTINGS->{daemon}       ||= $ENV{DANCER_DAEMON}       || 0;
-    $SETTINGS->{apphandler}   ||= $ENV{DANCER_APPHANDLER}   || 'Standalone';
-    $SETTINGS->{warnings}     ||= $ENV{DANCER_WARNINGS}     || 0;
-    $SETTINGS->{auto_reload}  ||= $ENV{DANCER_AUTO_RELOAD}  || 0;
-    $SETTINGS->{traces}       ||= $ENV{DANCER_TRACES}       || 0;
-    $SETTINGS->{logger}       ||= $ENV{DANCER_LOGGER}       || 'file';
-    $SETTINGS->{environment} ||=
+    $SETTINGS->{server}        ||= $ENV{DANCER_SERVER}        || '0.0.0.0';
+    $SETTINGS->{port}          ||= $ENV{DANCER_PORT}          || '3000';
+    $SETTINGS->{content_type}  ||= $ENV{DANCER_CONTENT_TYPE}  || 'text/html';
+    $SETTINGS->{charset}       ||= $ENV{DANCER_CHARSET}       || '';
+    $SETTINGS->{startup_info}  ||= !$ENV{DANCER_NO_STARTUP_INFO};
+    $SETTINGS->{daemon}        ||= $ENV{DANCER_DAEMON}        || 0;
+    $SETTINGS->{apphandler}    ||= $ENV{DANCER_APPHANDLER}    || 'Standalone';
+    $SETTINGS->{warnings}      ||= $ENV{DANCER_WARNINGS}      || 0;
+    $SETTINGS->{auto_reload}   ||= $ENV{DANCER_AUTO_RELOAD}   || 0;
+    $SETTINGS->{traces}        ||= $ENV{DANCER_TRACES}        || 0;
+    $SETTINGS->{server_tokens} ||= !$ENV{DANCER_NO_SERVER_TOKENS};
+    $SETTINGS->{logger}        ||= $ENV{DANCER_LOGGER}        || 'file';
+    $SETTINGS->{environment}   ||=
          $ENV{DANCER_ENVIRONMENT}
       || $ENV{PLACK_ENV}
       || 'development';
 
     setting $_ => {} for keys %MERGEABLE;
     setting template        => 'simple';
-    setting import_warnings => 1;
 }
 
 load_default_settings();
@@ -248,10 +272,16 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Config - how to configure Dancer to suit your needs
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 DESCRIPTION
 
 Dancer::Config handles reading and changing the configuration of your Dancer
@@ -276,6 +306,20 @@ exists in the application directory, it will be loaded, as a setting group.
 The same is done for the environment file located in the B<environments>
 directory.
 
+To fetch the available configuration values use the B<config> keyword that returns
+a reference to a hash:
+
+    my $port   = config->{port};
+    my $appdir = config->{appdir};
+
+By default, the module L<YAML> will be used to parse the configuration files.
+If desired, it is possible to use L<YAML::XS> instead by changing the YAML
+engine configuration in the application code:
+
+    config->{engines}{YAML}{module} = 'YAML::XS';
+
+See L<Dancer::Serializer::YAML> for more details.
+
 =head1 SUPPORTED SETTINGS
 
 =head2 Run mode and listening interface/port
@@ -359,7 +403,6 @@ have a lot of files being served in the B<public> folder that do not
 have an extension, and are text files, set the C<default_mime_type> to
 C<text/plain>.
 
-
 =head2 File / directory locations
 
 =head3 environment (string)
@@ -411,7 +454,6 @@ use Template Toolkit, add the following to C<config.yml>:
 
     template: template_toolkit
 
-
 =head3 layout (string)
 
 The name of the layout to use when rendering view. Dancer will look for
@@ -420,29 +462,49 @@ a matching template in the directory $views/layout.
 Your can override the default layout using the third argument of the
 C<template> keyword. Check C<Dancer> manpage for details.
 
-
 =head2 Logging, debugging and error handling
 
-=head3 import_warnings (boolean, default: enabled)
+=head3 strict_config (boolean, default: false)
 
-If true, or not present, C<use warnings> will be in effect in scripts in which
-you import C<Dancer>.  Set to a false value to disable this.
+If true, C<config> will return an object instead of a hash reference. See
+L<Dancer::Config::Object> for more information.
+
+=head3 global_warnings (boolean, default: false)
+
+If true, C<use warnings> will be in effect for all modules and scripts loaded
+by your Dancer application. Default is false.
 
 =head3 startup_info (boolean)
 
-If set to true, prints a banner at the server start with information such as
-versions and the environment (or "dancerfloor").
+If set to true (the default), prints a banner at server startup with information such as
+versions and the environment (or "dancefloor").
 
-Conforms to the environment variable DANCER_STARTUP_INFO.
+Can also be disabled by setting the environment variable DANCER_NO_STARTUP_INFO to
+a true value.
 
 =head3 warnings (boolean)
 
-If set to true, tells Dancer to consider all warnings as blocking errors.
+If set to true, tells Dancer to consider all warnings as blocking errors. Default is false.
 
 =head3 traces (boolean)
 
 If set to true, Dancer will display full stack traces when a warning or a die
-occurs. (Internally sets Carp::Verbose). Default to false.
+occurs. (Internally sets Carp::Verbose). Default is false.
+
+=head3 require_environment (boolean)
+
+If set to true, Dancer will fail during startup if your environment file is
+missing or can't be read. This is especially useful in production when you
+have things like memcached settings that need to be set per-environment.
+Default to false.
+
+=head3 server_tokens (boolean)
+
+If set to true (the default), Dancer will add an "X-Powered-By" header and also append
+the Dancer version to the "Server" header.
+
+Can also be disabled by setting the environment variable C<DANCER_NO_SERVER_TOKENS> to
+a true value.
 
 =head3 log_path (string)
 
@@ -480,6 +542,8 @@ B<core>, B<debug>, B<warning> or B<error>.
 
 =item B<debug> : all messages are logged
 
+=item B<info> : only info, warning and error messages are logged
+
 =item B<warning> : only warning and error messages are logged
 
 =item B<error> : only error messages are logged
@@ -490,7 +554,6 @@ During development, you'll probably want to use C<debug> to see your own debug
 messages, and C<core> if you need to see what Dancer is doing.  In production,
 you'll likely want C<error> or C<warning> only, for less-chatty logs.
 
-
 =head3 show_errors (boolean)
 
 If set to true, Dancer will render a detailed debug screen whenever an error is
@@ -502,7 +565,6 @@ The error screen attempts to sanitise sensitive looking information (passwords /
 card numbers in the request, etc) but you still should not have show_errors
 enabled whilst in production, as there is still a risk of divulging details.
 
-
 =head3 error_template (template path)
 
 This setting lets you specify a template to be used in case of runtime
@@ -524,7 +586,6 @@ The code throwing that error.
 
 =back
 
-
 =head3 auto_reload (boolean)
 
 Requires L<Module::Refresh> and L<Clone>.
@@ -573,6 +634,12 @@ This setting defaults to 1 and instructs the session cookie to be
 created with the C<HttpOnly> option active, meaning that JavaScript
 will not be able to access to its value.
 
+=head3 session_domain
+
+Allows you to set the domain property on the cookie, which will
+override the default.  This is useful for setting the session cookie's
+domain to something like C<.domain.com> so that the same cookie will
+be applicable and usable across subdomains of a base domain.
 
 =head2 auto_page (boolean)
 
@@ -592,10 +659,38 @@ Simply enable auto_page in your config:
 Then, if you request C</foo/bar>, Dancer will look in the views dir for
 C</foo/bar.tt>.
 
-Dancer will honor your C<before_template> code, and all default
+Dancer will honor your C<before_template_render> code, and all default
 variables. They will be accessible and interpolated on automatic
 served pages.
 
+The pages served this way will have C<Content-Type> set to C<text/html>,
+so don't use the feature for anything else.
+
+=head2 Route caching
+
+=head3 route_cache (boolean)
+
+If true, enables route caching (for quicker route resolution on larger apps - not caching
+of responses).  See L<Dancer::Route::Cache> for details. Default is false.
+
+=head3 route_cache_size_limit (bytes)
+
+Maximum size of route cache (e.g. 1024, 2M). Defaults to 10M (10MB) - see L<Dancer::Route::Cache>
+
+=head3 route_cache_path_limit (number)
+
+Maximum number of routes to cache. Defaults to 600 - see L<Dancer::Route::Cache>
+
+=head2 DANCER_CONFDIR and DANCER_ENVDIR
+
+It's possible to set the configuration directory and environment directory using this two
+environment variables. Setting `DANCER_CONFDIR` will have the same effect as doing
+
+    set confdir => '/path/to/confdir'
+
+and setting `DANCER_ENVDIR` will be similar to:
+
+    set envdir => '/path/to/environments'
 
 =head1 AUTHOR
 
@@ -611,4 +706,15 @@ itself.
 
 L<Dancer>
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
