@@ -1,4 +1,9 @@
 package Dancer::Cookie;
+BEGIN {
+  $Dancer::Cookie::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: class representing cookies
+$Dancer::Cookie::VERSION = '1.3131_0';
 use strict;
 use warnings;
 
@@ -29,7 +34,10 @@ sub to_header {
     my $value       = join('&', map {uri_escape($_)} $self->value);
     my $no_httponly = defined( $self->http_only ) && $self->http_only == 0;
 
-    my @headers = $self->name . '=' . $value;
+    my $name = $self->name;
+    $name =~ s/[=,; \t\r\n\013\014]//mg;
+
+    my @headers = $name . '=' . $value;
     push @headers, "path=" . $self->path        if $self->path;
     push @headers, "expires=" . $self->expires  if $self->expires;
     push @headers, "domain=" . $self->domain    if $self->domain;
@@ -118,10 +126,16 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Cookie - class representing cookies
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     use Dancer::Cookie;
@@ -163,7 +177,7 @@ understands...
 
 Months and years are currently fixed at 30 and 365 days.  This may change.
 
-Anything else is used verbatum.
+Anything else is used verbatim.
 
 =head2 domain
 
@@ -205,15 +219,13 @@ Creates a proper HTTP cookie header from the content.
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
