@@ -1,4 +1,9 @@
 package Dancer::Cookies;
+BEGIN {
+  $Dancer::Cookies::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: a singleton storage for all cookies
+$Dancer::Cookies::VERSION = '1.3131_0';
 use strict;
 use warnings;
 
@@ -36,11 +41,16 @@ sub parse_cookie_from_env {
         # a cookie string can contains something like:
         # cookie_name="foo=bar"
         # we want `cookie_name' as the value and `foo=bar' as the value
-        my( $name,$value ) = split(/\s*=\s*/, $cookie, 2);
+        my( $name, $value ) = split /\s*=\s*/, $cookie, 2;
+
+        # catch weird entries like 'cookie1=foo;;cookie2=bar'
+        next unless length $name;
+
         my @values;
-        if ( $value ne '' ) {
+        if ( defined $value && $value ne '' ) {
             @values = map { uri_unescape($_) } split( /[&;]/, $value );
         }
+
         $cookies->{$name} =
           Dancer::Cookie->new( name => $name, value => \@values );
     }
@@ -63,8 +73,7 @@ sub set_cookie {
 
 sub set_cookie_object {
     my ($class, $name, $cookie) = @_;
-    Dancer::SharedData->response->push_header(
-        'Set-Cookie' => $cookie->to_header);
+    Dancer::SharedData->response->add_cookie($name, $cookie);
     Dancer::Cookies->cookies->{$name} = $cookie;
 }
 
@@ -72,10 +81,18 @@ sub set_cookie_object {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Cookies - a singleton storage for all cookies
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     use Dancer::Cookies;
@@ -130,15 +147,13 @@ It also returns all the hashref it created.
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
