@@ -1,8 +1,13 @@
 package Dancer::Deprecation;
-
+BEGIN {
+  $Dancer::Deprecation::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: handle deprecation messages
+$Dancer::Deprecation::VERSION = '1.3131_0';
 use strict;
 use warnings;
-use Carp qw/croak carp/;
+use Carp;
+use Dancer::Exception qw(:all);
 
 sub deprecated {
     my ($class, %args) = @_;
@@ -25,16 +30,26 @@ sub deprecated {
     $msg .= " since version $deprecated_at" if defined $deprecated_at;
     $msg .= ". " . $args{reason} if defined $args{reason};
 
-    croak($msg) if $args{fatal};
+    raise core_deprecation => $msg if $args{fatal};
     carp($msg);
 }
 
 1;
 
+__END__
+
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Deprecation - handle deprecation messages
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
   Dancer::Deprecation->deprecated(
@@ -59,13 +74,13 @@ List of possible parameters:
 
 =item B<message> message to display
 
-=item B<fatal> if set to true, croak instead of carp
+=item B<fatal> if set to true, raises a Dancer::Exception (Core::Deprecation) instead of carp
 
-item B<reason> why is the feature deprecated
+=item B<reason> why is the feature deprecated
 
 =back
 
-You can call the method with no arguments, and a default message using informations from C<caller> will be build for you.
+You can call the method with no arguments, and a default message using information from C<caller> will be built for you.
 
 =head1 LICENSE
 
@@ -80,4 +95,15 @@ This module has been written by Alexis Sukrieh <sukria@sukria.net>
 
 L<Package::DeprecationManager>
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
