@@ -1,20 +1,26 @@
 package Dancer::Engine;
-
+BEGIN {
+  $Dancer::Engine::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: base class for Dancer engines
+$Dancer::Engine::VERSION = '1.3131_0';
 # This is the base-class of every engine abstract class.
 # This allow us to put in that single place the engine creation
 # from a namespace and a name, to its configuration initialization.
 
 use strict;
 use warnings;
+use Carp;
 use Dancer::ModuleLoader;
 use base 'Dancer::Object';
+use Dancer::Exception qw(:all);
 
 # constructor arguments:
 #      name     => $name_of_the_engine
 #      settings => $hash_of_engine_settings
 Dancer::Engine->attributes(qw(name type));
 
-# Accessor to the config hash, it may not be initialized if someone 
+# Accessor to the config hash, it may not be initialized if someone
 # creates a new engine without giving the appropriate arguments.
 # e.g. Dancer::Template::Simple->new();
 sub config {
@@ -28,12 +34,11 @@ sub config {
 # if Dancer::$type::$name exists.
 sub build {
     my ($class, $type, $name, $config) = @_;
-    
-    die "cannot build engine without type and name " 
-        unless $name and $type;
 
-    my $class_name = ucfirst($type);
-    my $namespace = "Dancer::${class_name}";
+    raise core_engine => "cannot build engine without type and name "
+      unless $name and $type;
+
+    my $class_name = $class->_engine_class($type);
 
     $config ||= {};
     $config->{engines} ||= {};
@@ -41,30 +46,59 @@ sub build {
 
     # trying to load the engine
     my $engine_class =
-      Dancer::ModuleLoader->class_from_setting($namespace => $name);
+      Dancer::ModuleLoader->class_from_setting($class_name => $name);
 
-    die "unknown $type engine '$name', " .
-        "perhaps you need to install $engine_class?"
-      unless Dancer::ModuleLoader->load($engine_class);
+    my( $loaded, $error ) = Dancer::ModuleLoader->load($engine_class);
+    $error = '' unless defined $error;
+
+    unless( $loaded ) {
+        my $tip = '';
+        if( $error =~ /Can't locate (\S+)\.pm in \@INC/ ) {
+            my $module = $1;
+            $module =~ s#/#::#g;
+            $tip = " (perhaps you need to install $module?)";
+        }
+
+        $error = ": $error" if length $error;
+        raise core_engine => "unable to load $type engine '$name'$tip$error";
+    }
 
     # creating the engine
     return $engine_class->new(
-        name     => $name,
-        type     => $type,
-        config   => $settings,
+        name   => $name,
+        type   => $type,
+        config => $settings,
     );
 }
 
+sub _engine_class {
+    my ($class, $type) = @_;
+    $type = ucfirst($type);
+    return "Dancer::${type}";
+}
+
+sub engine {
+    my ($class, $type) = @_;
+    return $class->_engine_class($type)->engine();
+}
+
+
 1;
 
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Engine - base class for Dancer engines
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     my $engine = Dancer::Engine->build( Serializer => 'JSON', $configuration );
@@ -113,15 +147,13 @@ Builds and returns the engine.
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
