@@ -1,8 +1,13 @@
 package Dancer::Error;
-
+BEGIN {
+  $Dancer::Error::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: class for representing fatal errors
+$Dancer::Error::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
+use Scalar::Util qw(blessed);
 
 use base 'Dancer::Object';
 
@@ -14,6 +19,7 @@ use Dancer::Factory::Hook;
 use Dancer::Session;
 use Dancer::FileUtils qw(open_file);
 use Dancer::Engine;
+use Dancer::Exception qw(:all);
 
 Dancer::Factory::Hook->instance->install_hooks(
     qw/before_error_render after_error_render before_error_init/);
@@ -116,19 +122,24 @@ sub dumper {
 
 
     # Take a copy of the data, so we can mask sensitive-looking stuff:
-    my %data     = Dancer::ModuleLoader->load('Clone') ?
-                   %{ Clone::clone($obj) }             :
-                   %$obj;
-    my $censored = _censor(\%data);
+    my $data     = Dancer::ModuleLoader->load('Clone') ?
+                   Clone::clone($obj)
+                   : eval Data::Dumper->new([$obj])->Purity(1)->Terse(1)->Deepcopy(1)->Dump;
+
+    $data = {%$data} if blessed($data); 
+
+	my $censored = _censor($data);
 
     #use Data::Dumper;
-    my $dd = Data::Dumper->new([\%data]);
-    $dd->Terse(1)->Quotekeys(0)->Indent(1);
+    my $dd = Data::Dumper->new([$data]);
+    $dd->Terse(1)->Quotekeys(0)->Indent(1)->Sortkeys(1);
     my $content = $dd->Dump();
     $content =~ s{(\s*)(\S+)(\s*)=>}{$1<span class="key">$2</span>$3 =&gt;}g;
     if ($censored) {
         $content
-          .= "\n\nNote: Values of $censored sensitive-looking keys hidden\n";
+            .= "\n\nNote: Values of $censored sensitive-looking key"
+            . ($censored == 1 ? '' : 's')
+            . " hidden\n";
     }
     return $content;
 }
@@ -136,7 +147,17 @@ sub dumper {
 # Given a hashref, censor anything that looks sensitive.  Returns number of
 # items which were "censored".
 sub _censor {
-    my $hash = shift;
+    my ( $hash, $recursecount ) = @_;
+    $recursecount ||= 0;
+
+    # we're checking recursion ourselves, no need to warn
+    no warnings 'recursion';
+
+    if ( $recursecount++ > 100 ) {
+        warn "Data exceeding 100 levels, truncating\n";
+        return $hash;
+    }
+
     if (!$hash || ref $hash ne 'HASH') {
         carp "_censor given incorrect input: $hash";
         return;
@@ -145,9 +166,9 @@ sub _censor {
     my $censored = 0;
     for my $key (keys %$hash) {
         if (ref $hash->{$key} eq 'HASH') {
-            $censored += _censor($hash->{$key});
+            $censored += _censor( $hash->{$key}, $recursecount );
         }
-        elsif ($key =~ /(pass|card?num|pan|secret)/i) {
+        elsif ($key =~ /(pass|card?num|pan|secret|private_key)/i) {
             $hash->{$key} = "Hidden (looks potentially sensitive)";
             $censored++;
         }
@@ -174,7 +195,16 @@ sub render {
 
     my $serializer = setting('serializer');
     Dancer::Factory::Hook->instance->execute_hooks('before_error_render', $self);
-    my $response = $serializer ? $self->_render_serialized() : $self->_render_html();
+    my $response;
+    try {
+        $response = $serializer ? $self->_render_serialized() : $self->_render_html();
+    } continuation {
+        my ($continuation) = @_;
+        # If we have a Route continuation, run the after hook, then
+        # propagate the continuation
+        Dancer::Factory::Hook->instance->execute_hooks('after_error_render', $response);
+        $continuation->rethrow();
+    };
     Dancer::Factory::Hook->instance->execute_hooks('after_error_render', $response);
     $response;
 }
@@ -184,8 +214,15 @@ sub _render_serialized {
 
     my $message =
       !ref $self->message ? {error => $self->message} : $self->message;
-    ref $message eq 'HASH' && defined $self->exception
-      and $message->{exception} = $self->exception;
+
+    if (ref $message eq 'HASH' && defined $self->exception) {
+        if (blessed($self->exception)) {
+            $message->{exception} = ref($self->exception);
+            $message->{exception} =~ s/^Dancer::Exception:://;
+        } else {
+            $message->{exception} = $self->exception;
+        }
+    }
 
     if (setting('show_errors')) {
         Dancer::Response->new(
@@ -284,10 +321,16 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Error - class for representing fatal errors
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     # taken from send_file:
@@ -298,7 +341,7 @@ Dancer::Error - class for representing fatal errors
         message => "No such file: `$path'"
     );
 
-    Dancer::Response->set($error->render);
+    $error->render;
 
 =head1 DESCRIPTION
 
@@ -328,6 +371,14 @@ The message of the error page.
 
 This is only an attribute getter, you'll have to set it at C<new>.
 
+=head2 exception
+
+The exception that caused the error. If the error was not caused by an
+exception, returns undef. Exceptions are usually objects that inherits of
+Dancer::Exception.
+
+This is only an attribute getter, you'll have to set it at C<new>.
+
 =head1 METHODS/SUBROUTINES
 
 =head2 new
@@ -350,6 +401,10 @@ The code that caused the error.
 
 The message that will appear to the user.
 
+=head3 exception
+
+The exception that will be useable by the rendering.
+
 =head2 backtrace
 
 Create a backtrace of the code where the error is caused.
@@ -357,7 +412,7 @@ Create a backtrace of the code where the error is caused.
 This method tries to find out where the error appeared according to the actual
 error message (using the C<message> attribute) and tries to parse it (supporting
 the regular/default Perl warning or error pattern and the L<Devel::SimpleTrace>
-output) and then returns an error-higlighted C<message>.
+output) and then returns an error-highlighted C<message>.
 
 =head2 tabulate
 
@@ -379,7 +434,7 @@ C<get_caller>), the settings and environment (using C<dumper>) and more.
 
 =head2 get_caller
 
-Creates a strack trace of callers.
+Creates a stack trace of callers.
 
 =head2 _censor
 
@@ -395,15 +450,13 @@ FIXME : this is not true anymore, output can be any charset. Need fixing.
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
