@@ -1,10 +1,14 @@
 package Dancer::Factory::Hook;
-
+BEGIN {
+  $Dancer::Factory::Hook::AUTHORITY = 'cpan:SUKRIA';
+}
+$Dancer::Factory::Hook::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
 
 use base 'Dancer::Object::Singleton';
+use Dancer::Exception qw(:all);
 
 __PACKAGE__->attributes(qw/ hooks registered_hooks/);
 
@@ -19,12 +23,12 @@ sub install_hooks {
     my ( $self, @hooks_name ) = @_;
 
     if ( !scalar @hooks_name ) {
-        croak "at least one name is required";
+        raise core_factory_hook => "at least one name is required";
     }
 
     foreach my $hook_name (@hooks_name) {
         if ( $self->hook_is_registered($hook_name) ) {
-            croak "$hook_name is already regsitered, please use another name";
+            raise core_factory_hook => "$hook_name is already regsitered, please use another name";
         }
         $self->_add_hook( $hook_name );
     }
@@ -53,22 +57,51 @@ sub hook_is_registered {
 sub execute_hooks {
     my ($self, $hook_name, @args) = @_;
 
-    croak("Can't ask for hooks without a position") unless $hook_name;
+    raise core_factory_hook => "Can't ask for hooks without a position" unless $hook_name;
 
     if (!$self->hook_is_registered($hook_name)){
-        croak("The hook '$hook_name' doesn't exists");
+        raise core_factory_hook => "The hook '$hook_name' doesn't exists";
     }
 
-   $_->(@args) foreach @{$self->get_hooks_for($hook_name)};
+   foreach my $h (@{$self->get_hooks_for($hook_name)}) {
+       $h->(@args);
+   }
 }
 
 sub get_hooks_for {
     my ( $self, $hook_name ) = @_;
 
-    croak("Can't ask for hooks without a position") unless $hook_name;
+    raise core_factory_hook => "Can't ask for hooks without a position" unless $hook_name;
 
     $self->hooks->{$hook_name} || [];
 }
 
 
 1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Dancer::Factory::Hook
+
+=head1 VERSION
+
+version 1.3131_0
+
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
