@@ -1,5 +1,8 @@
 package Dancer::GetOpt;
-
+BEGIN {
+  $Dancer::GetOpt::AUTHORITY = 'cpan:SUKRIA';
+}
+$Dancer::GetOpt::VERSION = '1.3131_0';
 use strict;
 use warnings;
 
@@ -45,7 +48,7 @@ sub print_usage {
    --daemon             Run in background (false)
    --port=XXXX          Port number to bind to (3000)
    --confdir=PATH       Path the config dir (appdir if not specified)
-   --environment=ENV    Environement to use (development)
+   --environment=ENV    Environment to use (development)
    --restart=1|0        Should we restart the application between each request
    --help               Display usage information
 
@@ -87,3 +90,30 @@ EOF
 }
 
 'Dancer::GetOpt';
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Dancer::GetOpt
+
+=head1 VERSION
+
+version 1.3131_0
+
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
