@@ -1,5 +1,9 @@
 package Dancer::Hook;
-
+BEGIN {
+  $Dancer::Hook::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: Class to manipulate hooks with Dancer
+$Dancer::Hook::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
@@ -57,20 +61,8 @@ sub new {
 
         Dancer::Logger::core( "entering " . $hook_name . " hook" );
 
-        
-        try { $code->(@arguments) }
-        catch {
-            my ($exception) = @_;
-            # exception is not a workflow continuation but a genuine error
-            my $err = Dancer::Error->new(
-                code    => 500,
-                title   => $hook_name . ' filter error',
-                message => "An error occured while executing the filter named $hook_name: $exception",
-                exception => $exception,
-            );
-            # raise a new halt exception
-            Dancer::halt( $err->render );
-        };
+        $code->(@arguments);
+
     };
 
     $self->properties($properties);
@@ -82,13 +74,19 @@ sub new {
 
 1;
 
+__END__
+
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Hook - Class to manipulate hooks with Dancer
 
-=head1 DESCRIPTION
+=head1 VERSION
 
-Manipulate hooks with Dancer
+version 1.3131_0
 
 =head1 SYNOPSIS
 
@@ -96,6 +94,10 @@ Manipulate hooks with Dancer
   use Dancer::Hook;
   Dancer::Hook->register_hooks_name(qw/before_auth after_auth/);
 
+=head1 DESCRIPTION
+
+Manipulate hooks with Dancer
+
 =head1 METHODS
 
 =head2 register_hook ($hook_name, [$properties], $code)
@@ -103,7 +105,7 @@ Manipulate hooks with Dancer
     hook 'before', {apps => ['main']}, sub {...};
 
     hook 'before' => sub {...};
-    
+
 Attaches a hook at some point, with a possible list of properties.
 
 Currently supported properties:
@@ -143,3 +145,16 @@ This module has been written by Alexis Sukrieh and others.
 
 This module is free software and is published under the same
 terms as Perl itself.
+
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
