@@ -1,5 +1,9 @@
 package Dancer::Logger::Abstract;
-
+BEGIN {
+  $Dancer::Logger::Abstract::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: Abstract logging engine for Dancer
+$Dancer::Logger::Abstract::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
@@ -77,6 +81,7 @@ sub format_message {
             return "[" . strftime( $block, localtime ) . "]";
         }
         elsif ( $type eq 'h' ) {
+            return '-' unless defined $r;
             return scalar $r->header($block) || '-';
         }
         else {
@@ -88,7 +93,7 @@ sub format_message {
     my $chars_mapping = {
         h => sub {
             defined $r
-              ? $r->env->{'HTTP_X_REAL_IP'} || $r->env->{'REMOTE_ADDR'}
+              ? $r->env->{'HTTP_X_REAL_IP'} || $r->env->{'REMOTE_ADDR'} || '-'
               : '-';
         },
         t => sub { Encode::decode(setting('charset') || 'utf8',
@@ -144,10 +149,18 @@ sub error   { $_[0]->_should('error')   and $_[0]->_log('error',   $_[1]) }
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Logger::Abstract - Abstract logging engine for Dancer
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
 In your configuration file:
@@ -285,15 +298,13 @@ warning, error).
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
