@@ -1,4 +1,9 @@
 package Dancer::Logger::Diag;
+BEGIN {
+  $Dancer::Logger::Diag::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: Test::More diag() logging engine for Dancer
+$Dancer::Logger::Diag::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use base 'Dancer::Logger::Abstract';
@@ -21,10 +26,18 @@ sub _log {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Logger::Diag - Test::More diag() logging engine for Dancer
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
 =head1 DESCRIPTION
@@ -43,19 +56,17 @@ lazily.
 
 =head2 _log
 
-Use Test::More's diag() to poutput the log message.
+Use Test::More's diag() to output the log message.
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
