@@ -1,11 +1,14 @@
 package Dancer::MIME;
-
+BEGIN {
+  $Dancer::MIME::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: Singleton object to handle MimeTypes
+$Dancer::MIME::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use base 'Dancer::Object::Singleton';
 
 use Dancer::Config;
-use Dancer::Deprecation;
 
 use Carp;
 use MIME::Types;
@@ -41,9 +44,9 @@ sub add_type {
 
 sub add_alias {
     my($self, $alias, $orig) = @_;
-    my $type = $self->for_name($orig) || croak "Can't find a mime type for $orig to alias $alias to";
+    my $type = $self->for_name($orig);
     $self->add_type($alias, $type);
-    return;
+    return $type;
 }
 
 sub for_file {
@@ -62,34 +65,25 @@ sub name_or_type {
 
 sub for_name {
     my ($self, $name) = @_;
-    return $self->custom_types->{$name} || $self->mime_type->mimeTypeOf(lc $name) || $self->default;
-}
-
-sub add_mime_type {
-    Dancer::Deprecation->deprecated(feature => 'add_mime_type',
-                                    fatal => 1,
-                                    reason => 'use the new "add" method');
-}
-
-sub add_mime_alias {
-    Dancer::Deprecation->deprecated(feature => 'add_mime_alias',
-                                    fatal => 1,
-                                    reason => 'use the new "add_alias" method');
-}
-
-sub mime_type_for {
-    Dancer::Deprecation->deprecated(feature => 'mime_type_for',
-                                    fatal => 1,
-                                    reason => 'use the new "name_or_type" method');
+    return $self->custom_types->{lc $name} || $self->mime_type->mimeTypeOf(lc $name) || $self->default;
 }
 
 42;
 
+__END__
+
+=pod
+
+=encoding UTF-8
 
 =head1 NAME
 
 Dancer::MIME - Singleton object to handle MimeTypes
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     # retrieve object instance
@@ -165,20 +159,6 @@ Retrieve the full hash table of added mime types.
 Resolves the $thing into a content $type whether it's the name of a
 MIME type like "txt" or already a mime type like "text/plain".
 
-=head1 DEPRECATED API
-
-=head2 add_mime_type
-
-Check the new C<add> method.
-
-=head2 add_mime_alias
-
-Check the new C<add> method.
-
-=head2 mime_type_for
-
-Check the new C<for_name> and C<name_or_type> methods.
-
 =head1 AUTHORS
 
 This module has been written and rewritten by different people from
@@ -192,5 +172,15 @@ This module is released under the same terms as Perl itself.
 
 L<Dancer>
 
-=cut
+=head1 AUTHOR
 
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
