@@ -1,48 +1,41 @@
 package Dancer::ModuleLoader;
-
+BEGIN {
+  $Dancer::ModuleLoader::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: dynamic module loading helpers for Dancer core components
+$Dancer::ModuleLoader::VERSION = '1.3131_0';
 # Abstraction layer for dynamic module loading
 
 use strict;
 use warnings;
+use Module::Runtime qw/ use_module /;
 
 sub load {
     my ($class, $module, $version) = @_;
-    # 0 is a valid version, so testing trueness of $version is not enough
-    if (defined $version && length $version) {
-        my ($res, $error) = $class->load_with_params($module);
-        $res or return wantarray ? (0, $error) : 0;
-        local $@;
-        eval { $module->VERSION($version) };
-        $error = $@;
-        $error and return wantarray ? (0, $error) : 0;
-        return 1;
-    }
-    # normal 'use', can be done via require + import
-    my ($res, $error) = $class->load_with_params($module);
+
+    my ($res, $error) = $class->require($module, $version);
     return wantarray ? ($res, $error) : $res;
 }
 
 sub require {
-    my ($class, $module) = @_;
-    local $@;
-    my $module_filename = $module;
-    $module_filename =~ s!::|'!/!g;
-    $module_filename .= '.pm';
-    eval { require $module_filename };
-    my $error = $@;
-    $error and return wantarray ? (0, $error) : 0;
-    return 1;
+    my ($class, $module, $version) = @_;
+    eval { defined $version ? use_module( $module, $version ) 
+                            : use_module( $module ) } 
+        or return wantarray ? (0, $@) : 0;
+    return 1; #success
 }
 
 sub load_with_params {
     my ($class, $module, @args) = @_;
     my ($res, $error) = $class->require($module);
     $res or return wantarray ? (0, $error) : 0;
+
     # From perlfunc : If no "import" method can be found then the call is
     # skipped, even if there is an AUTOLOAD method.
     if ($module->can('import')) {
+
         # bump Exporter Level to import symbols in the caller
-        local $Exporter::ExportLevel = ( $Exporter::ExportLevel || 0 ) + 1;
+        local $Exporter::ExportLevel = ($Exporter::ExportLevel || 0) + 1;
         local $@;
         eval { $module->import(@args) };
         my $error = $@;
@@ -75,10 +68,18 @@ sub class_from_setting {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::ModuleLoader - dynamic module loading helpers for Dancer core components
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
 Taken directly from Dancer::Template::TemplateToolkit (which is core):
@@ -108,7 +109,7 @@ please simply "C<use ModuleYouNeed>" in your code and don't use this module.
 
 =head2 load
 
-Runs a "C<use ModuleYouNeed>".
+Runs something like "C<use ModuleYouNeed>" at runtime.
 
     use Dancer::ModuleLoader;
     ...
@@ -128,7 +129,7 @@ Takes in arguments the module name, and optionally the minimum version number re
 In scalar context, returns 1 if successful, 0 if not.
 In list context, returns 1 if successful, C<(0, "error message")> if not.
 
-If you need to give argumentto the loading module, please use the method C<load_with_params>
+If you need to give argument to the loading module, please use the method C<load_with_params>
 
 =head2 require
 
@@ -219,15 +220,13 @@ L<Module::Load>, L<Module::New::Loader>
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
