@@ -1,11 +1,16 @@
 package Dancer::Object;
-
+BEGIN {
+  $Dancer::Object::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: Objects base class for Dancer
+$Dancer::Object::VERSION = '1.3131_0';
 # This class is a root class for each object in Dancer.
 # It provides basic OO tools for Perl5 without being... Moose ;-)
 
 use strict;
 use warnings;
 use Carp;
+use Dancer::Exception qw(:all);
 
 # constructor
 sub new {
@@ -18,7 +23,7 @@ sub new {
 
 sub clone {
     my ($self) = @_;
-    croak "The 'Clone' module is needed"
+    raise core => "The 'Clone' module is needed"
         unless Dancer::ModuleLoader->load('Clone');
     return Clone::clone($self);
 }
@@ -31,7 +36,7 @@ my $_attrs_per_class = {};
 sub get_attributes {
     my ($class, $visited_parents) = @_;
     # $visited_parents keeps track of parent classes we already handled, to
-    # avoid infinite recursion (in case of dependancies loop). It's not stored as class singleton, otherwise
+    # avoid infinite recursion (in case of dependencies loop). It's not stored as class singleton, otherwise
     # get_attributes wouldn't be re-entrant.
     $visited_parents ||= {};
     my @attributes = @{$_attrs_per_class->{$class} || [] };
@@ -99,10 +104,18 @@ sub attributes_defaults {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Object - Objects base class for Dancer
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     package My::Dancer::Extension;
@@ -181,15 +194,13 @@ default values. Usually called from within an C<init> function.
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
