@@ -1,11 +1,16 @@
 package Dancer::Object::Singleton;
-
+BEGIN {
+  $Dancer::Object::Singleton::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: Singleton base class for Dancer
+$Dancer::Object::Singleton::VERSION = '1.3131_0';
 # This class is a root class for singleton objects in Dancer.
 # It provides basic OO singleton tools for Perl5 without being... MooseX::Singleton ;-)
 
 use strict;
 use warnings;
 use Carp;
+use Dancer::Exception qw(:all);
 
 use base qw(Dancer::Object);
 
@@ -15,12 +20,12 @@ my %instances;
 # constructor
 sub new {
     my ($class) = @_;
-    croak "you can't call 'new' on $class, as it's a singleton. Try to call 'instance'";
+    raise core => "you can't call 'new' on $class, as it's a singleton. Try to call 'instance'";
 }
 
 sub clone {
     my ($class) = @_;
-    croak "you can't call 'clone' on $class, as it's a singleton. Try to call 'instance'";
+    raise core => "you can't call 'clone' on $class, as it's a singleton. Try to call 'instance'";
 }
 
 sub instance {
@@ -61,10 +66,18 @@ sub _setter_code {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Object::Singleton - Singleton base class for Dancer
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     package My::Dancer::Extension;
@@ -87,7 +100,7 @@ Dancer::Object::Singleton - Singleton base class for Dancer
 
 =head1 DESCRIPTION
 
-Dancer::Object::Singleton is meantto be used instead of Dancer::Object, if you
+Dancer::Object::Singleton is meant to be used instead of Dancer::Object, if you
 want your object to be a singleton, that is, a class that has only one instance
 in the application.
 
@@ -117,15 +130,13 @@ C<get_attributes>.
 
 =head1 AUTHOR
 
-Damien Krotkine
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2010 Damien Krotkine.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
