@@ -1,4 +1,9 @@
 package Dancer::Plugin;
+BEGIN {
+  $Dancer::Plugin::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: helper for writing Dancer plugins
+$Dancer::Plugin::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
@@ -6,6 +11,9 @@ use Carp;
 use base 'Exporter';
 use Dancer::Config 'setting';
 use Dancer::Hook;
+use Dancer::Factory::Hook;
+use Dancer::Exception qw(:all);
+
 use base 'Exporter';
 use vars qw(@EXPORT);
 
@@ -14,6 +22,10 @@ use vars qw(@EXPORT);
   register
   register_plugin
   plugin_setting
+  register_hook
+  execute_hooks
+  execute_hook
+  plugin_args
 );
 
 sub register($&);
@@ -22,6 +34,8 @@ my $_keywords = {};
 
 sub add_hook { Dancer::Hook->new(@_) }
 
+sub plugin_args { (undef, @_) }
+
 sub plugin_setting {
     my $plugin_orig_name = caller();
     (my $plugin_name = $plugin_orig_name) =~ s/Dancer::Plugin:://;
@@ -29,12 +43,27 @@ sub plugin_setting {
     return setting('plugins')->{$plugin_name} ||= {};
 }
 
+sub register_hook {
+    Dancer::Factory::Hook->instance->install_hooks(@_);
+}
+
+sub execute_hooks {
+    Dancer::Deprecation->deprecated(reason => "use 'execute_hook'",
+                                    version => '1.3098',
+                                    fatal => 0);
+    Dancer::Factory::Hook->instance->execute_hooks(@_);
+}
+
+sub execute_hook {
+    Dancer::Factory::Hook->instance->execute_hooks(@_);
+}
+
 sub register($&) {
     my ($keyword, $code) = @_;
     my $plugin_name = caller();
 
     $keyword =~ /^[a-zA-Z_]+[a-zA-Z0-9_]*$/
-      or croak "You can't use '$keyword', it is an invalid name"
+      or raise core_plugin => "You can't use '$keyword', it is an invalid name"
         . " (it should match ^[a-zA-Z_]+[a-zA-Z0-9_]*$ )";
 
     if (
@@ -42,11 +71,11 @@ sub register($&) {
         map  { s/^(?:\$|%|&|@|\*)//; $_ } 
         (@Dancer::EXPORT, @Dancer::EXPORT_OK)
     ) {
-        croak "You can't use '$keyword', this is a reserved keyword";
+        raise core_plugin => "You can't use '$keyword', this is a reserved keyword";
     }
     while (my ($plugin, $keywords) = each %$_keywords) {
         if (grep { $_->[0] eq $keyword } @$keywords) {
-            croak "You can't use $keyword, "
+            raise core_plugin => "You can't use $keyword, "
                 . "this is a keyword reserved by $plugin";
         }
     }
@@ -63,7 +92,8 @@ sub register_plugin {
     {
         no strict 'refs';
         # tried to use unshift, but it yields an undef warning on $plugin (perl v5.12.1)
-        @{"${plugin}::ISA"} = ('Exporter', 'Dancer::Plugin', @{"${plugin}::ISA"});
+        @{"${plugin}::ISA"} = ('Dancer::Plugin', @{"${plugin}::ISA"});
+        # this works because Dancer::Plugin already ISA Exporter
         push @{"${plugin}::EXPORT"}, @symbols;
     }
     return 1;
@@ -83,17 +113,20 @@ sub set_plugin_symbols {
 }
 
 1;
+
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Plugin - helper for writing Dancer plugins
 
-=head1 DESCRIPTION
+=head1 VERSION
 
-Create plugins for Dancer
+version 1.3131_0
 
 =head1 SYNOPSIS
 
@@ -123,6 +156,10 @@ And in your application:
 
     block_links_from; # this is exported by the plugin
 
+=head1 DESCRIPTION
+
+Create plugins for Dancer
+
 =head1 PLUGINS
 
 You can extend Dancer by writing your own Plugin.
@@ -155,6 +192,22 @@ module).
 
 A Dancer plugin inherits from Dancer::Plugin and Exporter transparently.
 
+=item B<register_hook>
+
+Allows a plugin to declare a list of supported hooks. Any hook declared like so
+can be executed by the plugin with C<execute_hooks>.
+
+    register_hook 'foo'; 
+    register_hook 'foo', 'bar', 'baz'; 
+
+=item B<execute_hooks>
+
+Allows a plugin to execute the hooks attached at the given position
+
+    execute_hooks 'some_hook';
+
+The hook must have been registered by the plugin first, with C<register_hook>.
+
 =item B<plugin_setting>
 
 Configuration for plugin should be structured like this in the config.yml of
@@ -176,6 +229,20 @@ for B<Dancer::Plugin::Foo::Bar>, use:
     "Foo::Bar":
       key: value
 
+=item B<plugin_args>
+
+To easy migration and interoperability between Dancer 1 and Dancer 2
+you can use this method to obtain the arguments or parameters passed
+to a plugin-defined keyword. Although not relevant for Dancer 1 only,
+or Dancer 2 only, plugins, it is useful for universal plugins.
+
+  register foo => sub {
+     my ($self, @args) = plugin_args(@_);
+     ...
+  }
+
+Note that Dancer 1 will return undef as the object reference.
+
 =back
 
 =head1 AUTHORS
@@ -187,4 +254,15 @@ This module has been written by Alexis Sukrieh and others.
 This module is free software and is published under the same
 terms as Perl itself.
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
