@@ -1,16 +1,21 @@
 package Dancer::Plugin::Ajax;
-
+BEGIN {
+  $Dancer::Plugin::Ajax::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: a plugin for adding Ajax route handlers
+$Dancer::Plugin::Ajax::VERSION = '1.3131_0';
 use strict;
 use warnings;
 
 use Dancer ':syntax';
+use Dancer::Exception ':all';
 use Dancer::Plugin;
 
 register 'ajax' => \&ajax;
 
 hook before => sub {
     if (request->is_ajax) {
-        content_type('text/xml');
+        content_type( plugin_setting->{content_type} || 'text/xml' );
     }
 };
 
@@ -29,7 +34,13 @@ sub ajax {
         # disable layout
         my $layout = setting('layout');
         setting('layout' => undef);
-        my $response = $code->();
+        my $response = try {
+            $code->();
+        } catch {
+            my $e = $_;
+            setting('layout' => $layout);
+            die $e;
+        };
         setting('layout' => $layout);
         return $response;
     };
@@ -55,10 +66,16 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Plugin::Ajax - a plugin for adding Ajax route handlers
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     package MyWebApp;
@@ -81,22 +98,48 @@ The route handler code will be compiled to behave like the following:
 
 =over 4
 
-=item * 
+=item *
 
 Pass if the request header X-Requested-With doesnt equal XMLHttpRequest
 
-=item * 
+=item *
 
 Disable the layout
 
 =item *
 
-The action built is a POST request.
+The action built matches POST / GET requests.
 
 =back
 
+=head1 NAME
+
+Dancer::Plugin::Ajax - 
+
+=head1 CONFIGURATION
+
+By default the plugin will use a content-type of 'text/xml' but this can be overwritten
+with plugin setting 'content_type'.
+
+Here is example to use JSON:
+
+  plugins:
+    'Ajax':
+      content_type: 'application/json'
+
 =head1 AUTHOR
 
 This module has been written by Alexis Sukrieh <sukria@sukria.net>
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
