@@ -1,9 +1,13 @@
 package Dancer::Renderer;
-
+BEGIN {
+  $Dancer::Renderer::AUTHORITY = 'cpan:SUKRIA';
+}
+$Dancer::Renderer::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
 use HTTP::Headers;
+use HTTP::Date qw( str2time time2str );
 use Dancer::Route;
 use Dancer::HTTP;
 use Dancer::Cookie;
@@ -17,6 +21,7 @@ use Dancer::FileUtils qw(path path_or_empty dirname read_file_content open_file)
 use Dancer::SharedData;
 use Dancer::Logger;
 use Dancer::MIME;
+use Dancer::Exception qw(:all);
 
 Dancer::Factory::Hook->instance->install_hooks(
     qw/before after before_serializer after_serializer before_file_render after_file_render/
@@ -55,10 +60,12 @@ sub render_error {
 sub response_with_headers {
     my $response = Dancer::SharedData->response();
 
-    $response->{headers} ||= HTTP::Headers->new;
-    my $powered_by = "Perl Dancer " . $Dancer::VERSION;
-    $response->header('X-Powered-By' => $powered_by);
-    $response->header('Server'       => $powered_by);
+    if (Dancer::Config::setting('server_tokens')) {
+        $response->{headers} ||= HTTP::Headers->new;
+        my $powered_by = "Perl Dancer " . $Dancer::VERSION;
+        $response->header('X-Powered-By' => $powered_by);
+        $response->header('Server'       => $powered_by);
+    }
 
     return $response;
 }
@@ -95,7 +102,7 @@ sub get_action_response {
     my $app = ($handler && $handler->app) ? $handler->app : Dancer::App->current();
 
     # run the before filters, before "running" the route handler
-    Dancer::Factory::Hook->instance->execute_hooks('before');
+    Dancer::Factory::Hook->instance->execute_hooks('before', $handler);
 
     # recurse if something has changed
     my $MAX_RECURSIVE_LOOP = 10;
@@ -103,7 +110,7 @@ sub get_action_response {
         || ($method ne Dancer::SharedData->request->method))
     {
         if ($depth > $MAX_RECURSIVE_LOOP) {
-            croak "infinite loop detected, "
+            raise core_renderer => "infinite loop detected, "
               . "check your route/filters for "
               . $method . ' '
               . $path;
@@ -116,6 +123,7 @@ sub get_action_response {
     if (defined $response && (my $status = $response->status)) {
         if ($status == 302 || $status == 301) {
             $class->serialize_response_if_needed();
+            Dancer::Factory::Hook->instance->execute_hooks('after', $response);
             return $response;
         }
     }
@@ -126,8 +134,17 @@ sub get_action_response {
         return $class->serialize_response_if_needed() if defined $response && $response->exists;
         # else, get the route handler's response
         Dancer::App->current($handler->{app});
-        $handler->run($request);
-        $class->serialize_response_if_needed();
+        try {
+            $handler->run($request);
+            $class->serialize_response_if_needed();
+        } continuation {
+            my ($continuation) = @_;
+            # If we have a Route continuation, run the after hook, then
+            # propagate the continuation
+            my $resp = Dancer::SharedData->response();
+            Dancer::Factory::Hook->instance->execute_hooks('after', $resp);
+            $continuation->rethrow();
+        };
         my $resp = Dancer::SharedData->response();
         Dancer::Factory::Hook->instance->execute_hooks('after', $resp);
         return $resp;
@@ -137,12 +154,60 @@ sub get_action_response {
     }
 }
 
+sub render_autopage {
+    return unless Dancer::setting('auto_page');
+
+    my $request = Dancer::SharedData->request;
+    my $path = $request->path_info;
+
+    # See if we find a matching view for this request, if so, render it
+    my $viewpath = $path;
+    $viewpath =~ s{^/}{};
+    my $view = Dancer::engine('template')->view($viewpath) || '';
+
+    if ($view && -f $view) {
+        # A view exists for the path requested, go ahead and render it:
+        return _autopage_response($viewpath);
+    }
+
+    # Try appending "index" and looking again
+    $view = Dancer::engine('template')->view(
+        Dancer::FileUtils::path($viewpath, 'index')
+    )|| '';
+    Dancer::error("Looking for $viewpath/index - got $view");
+    if ($view && -f $view) {
+        return _autopage_response(
+            Dancer::FileUtils::path($viewpath, 'index')
+        );
+    }
+
+    return;
+}
+sub _autopage_response {
+    my $viewpath = shift;
+    my $response = Dancer::Response->new;
+    $response->status(200);
+    $response->content(
+        Dancer::template($viewpath)
+    );
+    $response->header( 'Content-Type' => 'text/html' );
+    return $response;
+}
+
 sub serialize_response_if_needed {
     my $response = Dancer::SharedData->response();
 
     if (Dancer::App->current->setting('serializer') && $response->content()){
         Dancer::Factory::Hook->execute_hooks('before_serializer', $response);
-        Dancer::Serializer->process_response($response);
+        try {
+            Dancer::Serializer->process_response($response);
+        } continuation {
+            my ($continuation) = @_;
+            # If we have a Route continuation, run the after hook, then
+            # propagate the continuation
+            Dancer::Factory::Hook->execute_hooks('after_serializer', $response);
+            $continuation->rethrow();
+        };
         Dancer::Factory::Hook->execute_hooks('after_serializer', $response);
     }
     return $response;
@@ -160,11 +225,15 @@ sub get_file_response {
 
     my $app = Dancer::App->current;
     # TODO: this should be later removed with a check whether the file exists
-    # and then returning a 404, path_or_empty should be removed
-    my $static_file = path_or_empty( $app->setting('public'), $path_info );
+    # and then returning a 404
+    my $public = defined $app->setting('public') ?
+                 $app->setting('public')         :
+                 '';
+
+    my $static_file = path( $public, $path_info );
 
     return if ( !$static_file
-        || index( $static_file, path( $app->setting('public') ) ) != 0 );
+        || index( $static_file, ( path($public) || '' ) ) != 0 );
 
     return Dancer::Renderer->get_file_response_for_path( $static_file, undef,
         $request->content_type );
@@ -177,11 +246,23 @@ sub get_file_response_for_path {
         Dancer::Factory::Hook->execute_hooks( 'before_file_render',
             $static_file );
 
+        my $response = Dancer::SharedData->response() || Dancer::Response->new();
+
+        # handle If-Modified-Since
+        my $last_modified = (stat $static_file)[9];
+        my $since = str2time(Dancer::SharedData->request->env->{HTTP_IF_MODIFIED_SINCE});
+        if( defined $since && $since >= $last_modified ) {
+            $response->status( 304 );
+            $response->content( '' );
+            return $response;
+        }
+
         my $fh = open_file( '<', $static_file );
         binmode $fh;
-        my $response = Dancer::SharedData->response() || Dancer::Response->new();
         $response->status($status) if ($status);
+        $response->header( 'Last-Modified' => time2str( $last_modified ) );
         $response->header('Content-Type' => (($mime && _get_full_mime_type($mime)) ||
+                                             Dancer::SharedData->request->content_type ||
                                              _get_mime_type($static_file)));
         $response->content($fh);
 
@@ -238,3 +319,30 @@ Powered by <a href="http://perldancer.org/">Dancer</a> <% version %>
 
 
 1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Dancer::Renderer
+
+=head1 VERSION
+
+version 1.3131_0
+
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
