@@ -1,5 +1,9 @@
 package Dancer::Response;
-
+BEGIN {
+  $Dancer::Response::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: Response object for Dancer
+$Dancer::Response::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
@@ -95,12 +99,8 @@ sub halt {
         Dancer::SharedData->response($content);
     }
     else {
-        # This also sets the Response as the current one (SharedData)
-        Dancer::Response->new(
-            status => ($self->status || 200),
-            content => $content,
-            halted => 1,
-        );
+        $self->content($content) if defined $content;
+        $self->{halted} = 1;
     }
 }
 
@@ -187,11 +187,20 @@ sub build_cookie_headers {
 }
 1;
 
+__END__
+
+=pod
+
+=encoding UTF-8
 
 =head1 NAME
 
 Dancer::Response - Response object for Dancer
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     # create a new response object
@@ -218,10 +227,10 @@ Dancer::Response - Response object for Dancer
     Dancer::Response->new(
         status  => 200,
         content => 'my content',
-        headers => HTTP::Headers->new(...),
+        headers => ['X-Foo' => 'foo-value', 'X-Bar' => 'bar-value'],
     );
 
-create and return a new L<Dancer::Response> object
+create and return a new Dancer::Response object
 
 =head2 current
 
@@ -231,7 +240,6 @@ return the current Dancer::Response object, and reset the object
 
 =head2 exists
 
-
     if ($response->exists) {
         ...
     }
@@ -260,7 +268,7 @@ set or get the content of the current response object
     $response->status(201);
     Dancer::SharedData->response->status(201);
 
-set or get the status of the current response object
+Set or get the status of the current response object.  The default status is 200.
 
 =head2 content_type
 
@@ -272,14 +280,14 @@ set or get the status of the current response object
     $response->content_type('application/json');
     Dancer::SharedData->response->content_type('application/json');
 
-set or get the status of the current response object
+Set or get the status of the current response object.
 
 =head2 pass
 
     $response->pass;
     Dancer::SharedData->response->pass;
 
-set the pass value to one for this response
+Set the pass value to one for this response.
 
 =head2 has_passed
 
@@ -291,13 +299,15 @@ set the pass value to one for this response
         ...
     }
 
-test if the pass value is set to true
+Test if the pass value is set to true.
 
-=head2 halt
+=head2 halt($content)
 
     Dancer::SharedData->response->halt();
     $response->halt;
 
+Stops the processing of the current request.  See L<Dancer/halt>.
+
 =head2 halted
 
     if (Dancer::SharedData->response->halted) {
@@ -308,6 +318,8 @@ test if the pass value is set to true
         ...
     }
 
+This flag will be true if the current response has been halted.
+
 =head2 header
 
     # set the header
@@ -318,20 +330,31 @@ test if the pass value is set to true
     my $header = $response->header('X-Foo');
     my $header = Dancer::SharedData->response->header('X-Foo');
 
-get or set the value of a header
+Get or set the value of a header.
 
 =head2 headers
 
-    $response->headers(HTTP::Headers->new(...));
-    Dancer::SharedData->response->headers(HTTP::Headers->new(...));
+    $response->headers('X-Foo' => 'fff', 'X-Bar' => 'bbb');
+    Dancer::SharedData->response->headers('X-Foo' => 'fff', 'X-Bar' => 'bbb');
 
-return the list of headers for the current response
+Return the list of headers for the current response.
 
 =head2 headers_to_array
 
     my $headers_psgi = $response->headers_to_array();
     my $headers_psgi = Dancer::SharedData->response->headers_to_array();
 
-this method is called before returning a PSGI response. It transforms the list of headers to an array reference.
+This method is called before returning a L<< PSGI >> response. It transforms the list of headers to an array reference.
 
+=head1 AUTHOR
 
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
