@@ -1,18 +1,19 @@
 package Dancer::Route::Cache;
-
+BEGIN {
+  $Dancer::Route::Cache::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: route caching mechanism for L<Dancer>
+$Dancer::Route::Cache::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
 
 use base 'Dancer::Object';
 
-use vars '$VERSION';
-
 use Dancer::Config 'setting';
 use Dancer::Error;
 use Dancer::Exception qw(:all);
 
-$VERSION = '0.01';
 Dancer::Route::Cache->attributes('size_limit', 'path_limit');
 
 # static
@@ -72,35 +73,39 @@ sub parse_size {
 }
 
 sub route_from_path {
-    my ($self, $method, $path) = @_;
+    my ($self, $method, $path, $app_name) = @_;
 
     $method && $path
       or raise core_route => "Missing method or path";
 
-    return $self->{'cache'}{$method}{$path} || undef;
+    $app_name = 'main' unless defined $app_name;
+
+    return $self->{'cache'}{$app_name}{$method}{$path} || undef;
 }
 
 sub store_path {
-    my ($self, $method, $path, $route) = @_;
+    my ($self, $method, $path, $route, $app_name) = @_;
 
     $method && $path && $route
       or raise core_route => "Missing method, path or route";
 
-    $self->{'cache'}{$method}{$path} = $route;
+    $app_name = 'main' unless defined $app_name;
 
-    push @{$self->{'cache_array'}}, [$method, $path];
+    $self->{'cache'}{$app_name}{$method}{$path} = $route;
+
+    push @{$self->{'cache_array'}}, [$method, $path, $app_name];
 
     if (my $limit = $self->size_limit) {
         while ($self->route_cache_size() > $limit) {
-            my ($method, $path) = @{shift @{$self->{'cache_array'}}};
-            delete $self->{'cache'}{$method}{$path};
+            my ($method, $path, $app_name) = @{shift @{$self->{'cache_array'}}};
+            delete $self->{'cache'}{$app_name}{$method}{$path};
         }
     }
 
     if (my $limit = $self->path_limit) {
         while ($self->route_cache_paths() > $limit) {
-            my ($method, $path) = @{shift @{$self->{'cache_array'}}};
-            delete $self->{'cache'}{$method}{$path};
+            my ($method, $path, $app_name) = @{shift @{$self->{'cache_array'}}};
+            delete $self->{'cache'}{$app_name}{$method}{$path};
         }
     }
 }
@@ -112,15 +117,20 @@ sub route_cache_size {
 
     use bytes;
 
-    foreach my $method (keys %cache) {
-        $size += length $method;
+    foreach my $app_name (keys %cache) {
+        $size += length $app_name;
 
-        foreach my $path (keys %{$cache{$method}}) {
-            $size += length $path;
-            $size += length $cache{$method}{$path};
+        foreach my $method (keys %{$cache{$app_name}}) {
+            $size += length $method;
+
+            foreach my $path (keys %{$cache{$app_name}{$method}}) {
+                $size += length $path;
+                $size += length $cache{$app_name}{$method}{$path};
+            }
         }
     }
 
+
     no bytes;
 
     return $size;
@@ -130,21 +140,30 @@ sub route_cache_paths {
     my $self = shift;
     my %cache = $self->{'cache'} ? %{$self->{'cache'}} : ();
 
-    return scalar map { keys %{$cache{$_}} } keys %cache;
+    return scalar map { keys %{$_} } map { values %{$cache{$_}} } keys %cache;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Route::Cache - route caching mechanism for L<Dancer>
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     my $cache = Dancer::Route::Cache->new(
-        path_limit => 300, # optional
+        path_limit => 300, # optional, defaults to 600 (routes to cache)
+        size_limit => 5M,  # optional, defaults to 10M (10MB)
     );
 
     # storing a path
@@ -161,7 +180,7 @@ the requested path to a route.
 A major drawback is that L<Dancer> has to go over the matching on every request,
 which (especially on CGI-based applications) can be very time consuming.
 
-The caching mechanism allows to cache some requests to specific routes (but
+The caching mechanism allows one to cache some requests to specific routes (but
 B<NOT> specific results) and run those routes on a specific path. This allows us
 to speed up L<Dancer> quite a lot.
 
@@ -210,12 +229,15 @@ size limit.
 =head2 route_cache_paths
 
 Returns all the paths in the cache. This is used to enforce the path limit.
+Please be careful if you use L<Plack::Builder/mount> and some applications -
+routes are linked with applications and same path may be in some applications
+but with different handlers!
 
 =head1 ATTRIBUTES
 
 =head2 size_limit($limit)
 
-Allows to set a size limit of the cache.
+Allows one to set a size limit of the cache.
 
 Returns the limit (post-set).
 
@@ -233,15 +255,13 @@ Returns the limit (post-set).
 
 =head1 AUTHOR
 
-Sawyer X
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2010 Sawyer X.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
