@@ -1,5 +1,8 @@
 package Dancer::Serializer::Dumper;
-
+BEGIN {
+  $Dancer::Serializer::Dumper::AUTHORITY = 'cpan:SUKRIA';
+}
+$Dancer::Serializer::Dumper::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
@@ -37,3 +40,30 @@ sub deserialize {
 sub content_type {'text/x-data-dumper'}
 
 1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Dancer::Serializer::Dumper
+
+=head1 VERSION
+
+version 1.3131_0
+
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
