@@ -1,5 +1,9 @@
 package Dancer::Session;
-
+BEGIN {
+  $Dancer::Session::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: session engine for the Dancer framework
+$Dancer::Session::VERSION = '1.3131_0';
 use strict;
 use warnings;
 
@@ -21,6 +25,8 @@ sub init {
 
 # retrieve or create a session for the client
 sub get_current_session {
+    shift;
+    my %p       = @_;
     my $sid     = engine->read_session_id;
     my $session = undef;
     my $class   = ref(engine);
@@ -33,18 +39,19 @@ sub get_current_session {
 
     # Generate a session cookie; we want to do this regardless of whether the
     # session is new or existing, so that the cookie expiry is updated.
-    engine->write_session_id($session->id);
+    engine->write_session_id($session->id)
+        unless $p{no_update};
 
     return $session;
 }
 
-sub get { get_current_session() }
+sub get { get_current_session(@_) }
 
 sub read {
     my ($class, $key) = @_;
     return unless $key;
     my $session = get_current_session();
-    return $session->{$key};
+    return $session->get_value($key);
 }
 
 sub write {
@@ -54,22 +61,29 @@ sub write {
     $key eq 'id' and croak 'Can\'t store to session key with name "id"';
 
     my $session = get_current_session();
-    $session->{$key} = $value;
+    $session->set_value($key, $value);
 
     # TODO : should be moved as an "after" filter
-    $session->flush;
+    $session->flush unless $session->is_lazy;
     return $value;
 }
 
 1;
+
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Session - session engine for the Dancer framework
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 DESCRIPTION
 
 This module provides support for server-side sessions for the L<Dancer> web
@@ -112,8 +126,8 @@ Retrieving that value later:
     my $foo = session 'foo';
 
 You can either look for an existing item in the session storage or modify one.
-Here is a simple example of two route handlers that implement basic C</login> 
-and C</home> actions using the session engine. 
+Here is a simple example of two route handlers that implement basic C</login>
+and C</home> actions using the session engine.
 
     post '/login' => sub {
         # look for params and authenticate the user
@@ -136,7 +150,6 @@ whether the user is logged in for each route handler; there's an example in the
 L<Dancer::Cookbook> showing how to use a before filter to check whether the user
 is logged in before all requests, and redirect to a login page if not.
 
-
 =head1 SUPPORTED ENGINES
 
 Dancer has a modular session engine that makes implementing new session backends
@@ -155,8 +168,8 @@ but maybe not the best for production needs.
 
 =item L<Dancer::Session::Simple>
 
-A very simple session backend, holding all session data in memory.  This means 
-that sessions are volatile, and no longer exist when the process exits.  This 
+A very simple session backend, holding all session data in memory.  This means
+that sessions are volatile, and no longer exist when the process exits.  This
 module is likely to be most useful for testing purposes, and of little use for
 production.
 
@@ -179,8 +192,8 @@ inside encrypted cookies (this engine doesn't use a server-side storage).
 
 =item L<Dancer::Session::Storable>
 
-This backend stores sessions on disc using Storable, which offers solid 
-performance and reliable serialisation of various data structures.
+This backend stores sessions on disc using Storable, which offers solid
+performance and reliable serialization of various data structures.
 
 =item L<Dancer::Session::MongoDB>
 
@@ -202,7 +215,6 @@ between a Dancer app and other Plack-based apps.
 Dancer::Session may depend on third-party modules, depending on the session
 engine used. See the session engine module for details.
 
-
 =head1 AUTHORS
 
 This module has been written by Alexis Sukrieh. See the AUTHORS file that comes
@@ -217,4 +229,15 @@ itself.
 
 See L<Dancer> for details about the complete framework.
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
