@@ -1,15 +1,19 @@
 package Dancer::Session::YAML;
-
+BEGIN {
+  $Dancer::Session::YAML::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: YAML-file-based session backend for Dancer
+$Dancer::Session::YAML::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
 use base 'Dancer::Session::Abstract';
 
-use Fcntl ':flock';
 use Dancer::Logger;
 use Dancer::ModuleLoader;
 use Dancer::Config 'setting';
-use Dancer::FileUtils qw(path set_file_mode);
+use Dancer::FileUtils qw(path atomic_write);
+use Dancer::Exception qw(:all);
 
 # static
 
@@ -20,7 +24,7 @@ sub init {
     $self->SUPER::init(@_);
 
     if (!keys %session_dir_initialized) {
-        croak "YAML is needed and is not installed"
+        raise core_session => "YAML is needed and is not installed"
           unless Dancer::ModuleLoader->load('YAML');
     }
 
@@ -34,7 +38,7 @@ sub init {
         # make sure session_dir exists
         if (!-d $session_dir) {
             mkdir $session_dir
-              or croak "session_dir $session_dir cannot be created";
+              or raise core_session => "session_dir $session_dir cannot be created";
         }
         Dancer::Logger::core("session_dir : $session_dir");
     }
@@ -64,7 +68,6 @@ sub retrieve {
     return unless -f $session_file;
 
     open my $fh, '+<', $session_file or die "Can't open '$session_file': $!\n";
-    flock $fh, LOCK_EX or die "Can't lock file '$session_file': $!\n";
     my $content = YAML::LoadFile($fh);
     close $fh or die "Can't close '$session_file': $!\n";
 
@@ -90,24 +93,27 @@ sub flush {
     my $self         = shift;
     my $session_file = yaml_file( $self->id );
 
-    open my $fh, '>', $session_file or die "Can't open '$session_file': $!\n";
-    flock $fh, LOCK_EX or die "Can't lock file '$session_file': $!\n";
-    set_file_mode($fh);
-    print {$fh} YAML::Dump($self);
-    close $fh or die "Can't close '$session_file': $!\n";
+    atomic_write( setting('session_dir'), yaml_file($self->id), YAML::Dump($self) );
 
     return $self;
 }
 
 1;
+
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Session::YAML - YAML-file-based session backend for Dancer
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 DESCRIPTION
 
 This module implements a session engine based on YAML files. Session are stored
@@ -137,12 +143,12 @@ files in /tmp/dancer-sessions
 
 =head2 reset
 
-to avoid checking if the sessions directory exists everytime a new session is
+To avoid checking if the sessions directory exists every time a new session is
 created, this module maintains a cache of session directories it has already
 created. C<reset> wipes this cache out, forcing a test for existence
 of the sessions directory next time a session is created. It takes no argument.
 
-This is particulary useful if you want to remove the sessions directory on the
+This is particularly useful if you want to remove the sessions directory on the
 system where your app is running, but you want this session engine to continue
 to work without having to restart your application.
 
@@ -168,4 +174,15 @@ This module is copyright (c) 2009 Alexis Sukrieh <sukria@sukria.net>
 This module is free software and is released under the same terms as Perl
 itself.
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
