@@ -1,9 +1,17 @@
 package Dancer::SharedData;
-
+BEGIN {
+  $Dancer::SharedData::AUTHORITY = 'cpan:SUKRIA';
+}
+$Dancer::SharedData::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Dancer::Timer;
 use Dancer::Response;
+use Dancer::Factory::Hook;
+
+Dancer::Factory::Hook->instance->install_hooks(
+    qw/on_reset_state/
+);
 
 # shared variables
 my $vars = {};
@@ -43,8 +51,11 @@ sub reset_timer { $_timer = Dancer::Timer->new }
 # purging accessor
 sub reset_all {
     my ($self, %options) = @_;
+    my $is_forward = exists($options{reset_vars}) && ! $options{reset_vars};
 
-    if (!exists($options{reset_vars}) || $options{reset_vars}) {
+    Dancer::Factory::Hook->execute_hooks('on_reset_state', $is_forward);
+
+    if (!$is_forward) {
         $vars = {};
     }
     undef $_request;
@@ -54,3 +65,30 @@ sub reset_all {
 }
 
 'Dancer::SharedData';
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Dancer::SharedData
+
+=head1 VERSION
+
+version 1.3131_0
+
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
