@@ -1,5 +1,9 @@
 package Dancer::Template::Abstract;
-
+BEGIN {
+  $Dancer::Template::Abstract::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: abstract class for Dancer's template engines
+$Dancer::Template::Abstract::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
@@ -205,14 +209,21 @@ sub template {
 sub view_exists { return defined $_[1] &&  -f $_[1] }
 
 1;
+
 __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Template::Abstract - abstract class for Dancer's template engines
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 DESCRIPTION
 
 This class is provided as a base class for each template engine. Any template
@@ -347,4 +358,15 @@ Examples :
 
 This module has been written by Alexis Sukrieh, see L<Dancer> for details.
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
