@@ -1,4 +1,9 @@
 package Dancer::Template::Simple;
+BEGIN {
+  $Dancer::Template::Simple::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: pure Perl 5 template engine for Dancer
+$Dancer::Template::Simple::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use Carp;
@@ -6,6 +11,7 @@ use Carp;
 use base 'Dancer::Template::Abstract';
 Dancer::Template::Simple->attributes('start_tag', 'stop_tag');
 use Dancer::FileUtils 'read_file_content';
+use Dancer::Exception qw(:all);
 
 sub init {
     my $self     = shift;
@@ -106,10 +112,10 @@ sub _read_content_from_template {
         $content = $$template;
     }
     else {
-        croak "'$template' is not a regular file"
+        raise core_template => "'$template' is not a regular file"
           unless -f $template;
         $content = read_file_content($template);
-        croak "unable to read content for file $template"
+        raise core_template => "unable to read content for file $template"
           if not defined $content;
     }
     return $content;
@@ -157,13 +163,19 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Template::Simple - pure Perl 5 template engine for Dancer
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 DESCRIPTION
 
-This template engine is provided as a default one for the Dancer micro
+This template engine is provided as a default one for the L<< Dancer >> micro
 framework.
 
 This template engine should be fine for development purposes but is not a
@@ -171,7 +183,7 @@ powerful one, it's written in pure Perl and has no C bindings to accelerate the
 template processing.
 
 If you want to power an application with Dancer in production environment, it's
-strongly advised to switch to Dancer::Template::TemplateToolkit.
+strongly advised to switch to L<< Dancer::Template::TemplateToolkit >>.
 
 =head1 SYNTAX
 
@@ -203,4 +215,15 @@ This module has been written by Alexis Sukrieh.
 This module is free software and is released under the same terms as Perl
 itself.
 
+=head1 AUTHOR
+
+Dancer Core Developers
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2010 by Alexis Sukrieh.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
