@@ -1,5 +1,9 @@
 package Dancer::Timer;
-
+BEGIN {
+  $Dancer::Timer::AUTHORITY = 'cpan:SUKRIA';
+}
+#ABSTRACT: a timer for Dancer
+$Dancer::Timer::VERSION = '1.3131_0';
 use strict;
 use warnings;
 use base 'Dancer::Object';
@@ -29,10 +33,18 @@ sub to_string {
 
 __END__
 
+=pod
+
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer::Timer - a timer for Dancer
 
+=head1 VERSION
+
+version 1.3131_0
+
 =head1 SYNOPSIS
 
     use Dancer::Timer;
@@ -84,15 +96,13 @@ Same as C<tick>, just more expressive.
 
 =head1 AUTHOR
 
-Alexis Sukrieh
+Dancer Core Developers
 
-=head1 LICENSE AND COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009-2010 Alexis Sukrieh.
+This software is copyright (c) 2010 by Alexis Sukrieh.
 
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See http://dev.perl.org/licenses/ for more information.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
+=cut
